#!/bin/bash

ECHO=/bin/echo
RM=/bin/rm
MKDIR=/bin/mkdir
CHOWN=/bin/chown
CHGRP=/bin/chgrp
CHMOD=/bin/chmod
SU=/bin/su
GREP=/bin/grep
CAT=/bin/cat
MAWK=/usr/bin/mawk
HEAD=/usr/bin/head
MYSQLDUMP=/usr/bin/mysqldump
PSQL=/usr/bin/psql
MYSQL=/usr/bin/mysql
RSYNC=/usr/bin/rsync
DPKG=/usr/bin/dpkg
DATE=/bin/date
DUPLICITY=/usr/bin/duplicity
MKTEMP=/bin/mktemp
TR=/usr/bin/tr
FIND=/usr/bin/find
SLEEP=/bin/sleep
PG_DROPCLUSTER=/usr/bin/pg_dropcluster
PG_CREATECLUSTER=/usr/bin/pg_createcluster
CHECK_PERMS=/usr/sbin/check_perms

function gnupanel_perms
{
    ${CHOWN} -R www-data:www-data /var/www
    ${FIND} /var/www -type d -exec ${CHMOD} 0700 {} \;
    ${FIND} /var/www -type f -exec ${CHMOD} 0600 {} \;

    ${CHOWN} -R correos:mail /var/mail/correos
    ${FIND} /var/mail/correos -type d -exec ${CHMOD} 2700 {} \;
    ${FIND} /var/mail/correos -type f -exec ${CHMOD} 600 {} \;
    ${CHECK_PERMS} -f
    ${CHECK_PERMS} -f
}

function restore_gnupanel_postgres
{
    LOCAL_RESTORE_DIR=$1
    GNUPANEL_INSTALL_DATA_FILE=/etc/gnupanel/GNUPANEL_INSTALL_DATA
    PG_ALL_DB_FILE=${LOCAL_RESTORE_DIR}/databases/postgresql/all_postgres.dump

    PG_TMP_FILE=`${MKTEMP} --suffix=.sql`

    GNUPANEL_PG=`${CAT} ${GNUPANEL_INSTALL_DATA_FILE} | ${GREP} GNUPANEL_PG | ${MAWK} -F ":" '{print $2;}'`
    PROFTPD_PG=`${CAT} ${GNUPANEL_INSTALL_DATA_FILE} | ${GREP} PROFTPD_PG | ${MAWK} -F ":" '{print $2;}'`
    APACHE_PG=`${CAT} ${GNUPANEL_INSTALL_DATA_FILE} | ${GREP} APACHE_PG | ${MAWK} -F ":" '{print $2;}'`
    POSTFIX_PG=`${CAT} ${GNUPANEL_INSTALL_DATA_FILE} | ${GREP} POSTFIX_PG | ${MAWK} -F ":" '{print $2;}'`
    PDNS_PG=`${CAT} ${GNUPANEL_INSTALL_DATA_FILE} | ${GREP} PDNS_PG | ${MAWK} -F ":" '{print $2;}'`
    SDNS_PG=`${CAT} /etc/gnupanel/sdns.conf.pl | ${GREP} pasaportedb | ${MAWK} -F "=" '{print $2;}' | ${TR} -d " \";"`

    ${ECHO} "GRANT ALL ON DATABASE gnupanel TO gnupanel;" > ${PG_TMP_FILE}
    ${ECHO} "GRANT CONNECT ON DATABASE gnupanel TO apache;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT CONNECT ON DATABASE gnupanel TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT CONNECT ON DATABASE gnupanel TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT CONNECT ON DATABASE gnupanel TO sdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT CONNECT ON DATABASE gnupanel TO proftpd;" >> ${PG_TMP_FILE}

    ${ECHO} "\connect gnupanel" >> ${PG_TMP_FILE}

    ${ECHO} "GRANT SELECT ON TABLE gnupanel_apache_user TO apache;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT ALL ON TABLE gnupanel_pdns_domains TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT ALL ON TABLE gnupanel_pdns_domains_nat TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT ALL ON TABLE gnupanel_pdns_records TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT ALL ON TABLE gnupanel_pdns_records_nat TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_pdns_supermasters TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_pdns_supermasters_nat TO pdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_alias TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_autoreply TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_mailuser TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_transport TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_transport TO sdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_transport_listas TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_virtual TO postfix;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_postfix_virtual TO sdns;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT ON TABLE gnupanel_proftpd_ftpgroup TO proftpd;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT INSERT,SELECT,UPDATE ON TABLE gnupanel_proftpd_ftpquotalimits TO proftpd;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT INSERT,SELECT,UPDATE ON TABLE gnupanel_proftpd_ftpquotatallies TO proftpd;" >> ${PG_TMP_FILE}
    ${ECHO} "GRANT SELECT,UPDATE ON TABLE gnupanel_proftpd_ftpuser TO proftpd;" >> ${PG_TMP_FILE}

    ${ECHO} "ALTER TABLE gnupanel_proftpd_ftpuser ALTER COLUMN uid SET DEFAULT 33;" >> ${PG_TMP_FILE}
    ${ECHO} "ALTER TABLE gnupanel_proftpd_ftpuser ALTER COLUMN gid SET DEFAULT 33;" >> ${PG_TMP_FILE}

    ${ECHO} "ALTER TABLE gnupanel_proftpd_ftpgroup ALTER COLUMN gid SET DEFAULT 33;" >> ${PG_TMP_FILE}

    ${ECHO} "UPDATE gnupanel_proftpd_ftpuser SET uid = 33, gid = 33 ;" >> ${PG_TMP_FILE}
    ${ECHO} "UPDATE gnupanel_proftpd_ftpgroup SET gid = 33;" >> ${PG_TMP_FILE}

    ${ECHO} "ALTER USER gnupanel ENCRYPTED PASSWORD '${GNUPANEL_PG}';" >> ${PG_TMP_FILE}
    ${ECHO} "ALTER USER apache ENCRYPTED PASSWORD '${APACHE_PG}';" >> ${PG_TMP_FILE}
    ${ECHO} "ALTER USER postfix ENCRYPTED PASSWORD '${POSTFIX_PG}';" >> ${PG_TMP_FILE}
    ${ECHO} "ALTER USER pdns ENCRYPTED PASSWORD '${PDNS_PG}';" >> ${PG_TMP_FILE}
    ${ECHO} "ALTER USER proftpd ENCRYPTED PASSWORD '${PROFTPD_PG}';" >> ${PG_TMP_FILE}
    ${ECHO} "ALTER USER sdns ENCRYPTED PASSWORD '${SDNS_PG}';" >> ${PG_TMP_FILE}
    ${ECHO} "UPDATE gnupanel_apacheconf SET estado = 0;" >> ${PG_TMP_FILE}

    POSTGRESQL_CONF=/etc/postgresql/${PG_VERSION}/main/postgresql.conf
    PG_HBA_CONF=/etc/postgresql/${PG_VERSION}/main/pg_hba.conf

    POSTGRESQL_CONF_TMP=`${MKTEMP}`
    PG_HBA_CONF_TMP=`${MKTEMP}`

    ${CAT} ${POSTGRESQL_CONF} > ${POSTGRESQL_CONF_TMP}
    ${CAT} ${PG_HBA_CONF} > ${PG_HBA_CONF_TMP}

    /etc/init.d/postgresql stop
    ${PG_DROPCLUSTER} ${PG_VERSION} main
    ${PG_CREATECLUSTER} -u postgres -g postgres --locale=C -e SQL_ASCII ${PG_VERSION} main
    ${CAT} ${POSTGRESQL_CONF_TMP} > ${POSTGRESQL_CONF}
    ${CAT} ${PG_HBA_CONF_TMP} > ${PG_HBA_CONF}
    /etc/init.d/postgresql start

    ${CHOWN} postgres:postgres ${PG_ALL_DB_FILE}
    ${CHOWN} postgres:postgres ${PG_TMP_FILE}

    ${CHMOD} 0444 ${PG_ALL_DB_FILE}
    ${CHMOD} 0444 ${PG_TMP_FILE}

    ${SU} postgres -c "${PSQL} -f ${PG_ALL_DB_FILE} postgres"
    ${SU} postgres -c "${PSQL} -f ${PG_TMP_FILE} postgres"

    /etc/init.d/gnupanel-transf restart

    ${RM} -f ${PG_TMP_FILE}
    ${RM} -f ${POSTGRESQL_CONF_TMP}
    ${RM} -f ${PG_HBA_CONF_TMP}

    SQL="SELECT count(id_apache) FROM gnupanel_apacheconf WHERE estado <> 9;"

    WAIT_CONF=`${SU} postgres -c "${PSQL} -d gnupanel -t -q -c \"${SQL}\" | ${TR} -d ' ' "`
    MAX_WAIT=0

    while ! [ "${WAIT_CONF}" = "0" ]
    do
	WAIT_CONF=`${SU} postgres -c "${PSQL} -d gnupanel -t -q -c \"${SQL}\" | ${TR} -d ' ' "`
	if [ ${MAX_WAIT} -gt 70 ]
	then
	    WAIT_CONF=0
	fi
	MAX_WAIT=$[ ${MAX_WAIT} + 1 ]
	${SLEEP} 1
    done
}

function restore_gnupanel_mysql
{
    LOCAL_RESTORE_DIR=$1
    GNUPANEL_INSTALL_DATA_FILE=/etc/gnupanel/GNUPANEL_INSTALL_DATA
    MY_ALL_DB_FILE=${LOCAL_RESTORE_DIR}/databases/mysql/all_mysql.sql

    MY_TMP_FILE=`${MKTEMP} --suffix=.sql`
    MY_TMP_FILE_SQL=`${MKTEMP} --suffix=.sql`

    MYSQL_PASSWD=`${CAT} ${GNUPANEL_INSTALL_DATA_FILE} | ${GREP} MYSQL_PASSWD | ${MAWK} -F ":" '{print $2;}'`

    MYSQL_DEBIAN_PW=`${CAT} /etc/mysql/debian.cnf | ${GREP} password | ${HEAD} -n 1 | ${MAWK} '{print $3;}'`

    ${ECHO} "GRANT ALL PRIVILEGES ON *.* to root IDENTIFIED BY '${MYSQL_PASSWD}' ;" > ${MY_TMP_FILE}
    ${ECHO} "GRANT ALL PRIVILEGES ON *.* to root@localhost IDENTIFIED BY '${MYSQL_PASSWD}' ;" >> ${MY_TMP_FILE}

    ${ECHO} "GRANT ALL PRIVILEGES ON *.* to 'debian-sys-maint' IDENTIFIED BY '${MYSQL_DEBIAN_PW}' ;" >> ${MY_TMP_FILE}
    ${ECHO} "GRANT ALL PRIVILEGES ON *.* to 'debian-sys-maint'@localhost IDENTIFIED BY '${MYSQL_DEBIAN_PW}' ;" >> ${MY_TMP_FILE}

    ${ECHO} "FLUSH PRIVILEGES ;" >> ${MY_TMP_FILE}

    ${CAT} ${MY_ALL_DB_FILE} > ${MY_TMP_FILE_SQL}
    ${CAT} ${MY_TMP_FILE} >> ${MY_TMP_FILE_SQL}

    ${MYSQL} -u root -p${MYSQL_PASSWD} < ${MY_TMP_FILE_SQL}

    ${RM} -f ${MY_TMP_FILE}
    ${RM} -f ${MY_TMP_FILE_SQL}
}

function restore_gnupanel_files
{
    LOCAL_RESTORE_DIR=$1
    DIR_DATA_RESTORE=${LOCAL_RESTORE_DIR}/files

    DIR_RESTORE=""

    for dir_data in ${DIRS}
    do
	if ! [ "${dir_data}" = "/etc" ]
	then
	    if [ "${dir_data}" = "/var/log" ]
	    then
		DIR_RESTORE=${dir_data}/apache2
	    else
		DIR_RESTORE=${dir_data}
	    fi
	    if ! [ -d ${DIR_TO_RESTORE}${DIR_RESTORE} ]
	    then
		${MKDIR} -p ${DIR_TO_RESTORE}${DIR_RESTORE}
	    fi
	    ${RSYNC} -q -a -h -P -r --bwlimit=0 -v --delete-before --force ${DIR_DATA_RESTORE}${DIR_RESTORE}/ ${DIR_RESTORE}/
	fi
    done
}

function backup_postgres
{
    dir_pg=$1/databases/postgresql

    if ! [ -d ${dir_pg} ]
    then
	${MKDIR} -p ${dir_pg}
    fi

    ${RM} -f ${dir_pg}/* 1>/dev/null 2>/dev/null
    ${CHOWN} postgres ${dir_pg}
    ${CHGRP} postgres ${dir_pg}

    ${ECHO} "BEGIN backup PostgreSQL"

    ${SU} postgres -c "/usr/lib/postgresql/${PG_VERSION}/bin/pg_dumpall -c --inserts > ${dir_pg}/all_postgres.sql"
    ${SU} postgres -c "/usr/lib/postgresql/${PG_VERSION}/bin/pg_dumpall > ${dir_pg}/all_postgres.dump"

    SQL="SELECT datname FROM pg_catalog.pg_database WHERE datname <> 'postgres' AND datname <> 'template0' AND datname <> 'template1' ;"
    PG_DATABASES=`${SU} postgres -c "${PSQL} postgres -t -q -c \"${SQL}\" " | ${MAWK} '{print $1;}' `

    for pg_database in ${PG_DATABASES}
    do
        ${SU} postgres -c "/usr/lib/postgresql/${PG_VERSION}/bin/pg_dump -c --inserts ${pg_database} > ${dir_pg}/${pg_database}_postgresql.sql"
        ${SU} postgres -c "/usr/lib/postgresql/${PG_VERSION}/bin/pg_dump -Fc ${pg_database} > ${dir_pg}/${pg_database}_postgresql.dump"
    done

    ${ECHO} "END backup PostgreSQL"
    cd /
}

function backup_mysql
{
    dir_my=$1/databases/mysql

    MYSQL_PW=

    if [ -e /etc/mysql/debian.cnf ]
    then
	MYSQL_PW=`${CAT} /etc/mysql/debian.cnf | ${GREP} password | ${HEAD} -n 1 | ${MAWK} '{print $3;}'`
    fi

    if ! [ -d ${dir_my} ]
    then
	${MKDIR} -p ${dir_my}
    fi

    ${RM} -f ${dir_my}/* 1>/dev/null 2>/dev/null

    ${ECHO} "BEGIN backup MySQL"
    ${MYSQLDUMP} -f --all-databases --password=${MYSQL_PW} --user=debian-sys-maint > ${dir_my}/all_mysql.sql
    ${MYSQLDUMP} -f --password=${MYSQL_PW} --user=debian-sys-maint phpmyadmin > ${dir_my}/phpmyadmin_mysql.sql
    ${MYSQLDUMP} -f --password=${MYSQL_PW} --user=debian-sys-maint roundcube > ${dir_my}/roundcube_mysql.sql

    SQL="SELECT nombre_base FROM gnupanel_bases_de_datos WHERE id_tipo_base = 1 ;"
    MY_DATABASES=`${SU} postgres -c "${PSQL} gnupanel -t -q -c \"${SQL}\" " | ${MAWK} '{print $1;}' `
    for my_database in ${MY_DATABASES}
    do
        ${MYSQLDUMP} -f --password=${MYSQL_PW} --user=debian-sys-maint ${my_database} > ${dir_my}/${my_database}_mysql.sql
    done

    ${ECHO} "END backup MySQL"
    cd /
}


CONF_FILE=/etc/gnupanel/gnupanel-backup.conf
LOG_FILE=/var/log/gnupanel-backup.sh.log
ERR_FILE=/var/log/gnupanel-backup.sh.err

${RM} -f ${LOG_FILE} 1>/dev/null 2>/dev/null
${RM} -f ${ERR_FILE} 1>/dev/null 2>/dev/null

if [ -f ${CONF_FILE} ]
then
    . ${CONF_FILE}
else
    ${ECHO} "Configuration file ${CONF_FILE} not exists"
    exit 1
fi

DUP_SCHEME=ftp

DATE_BACKUP=`${DATE}`

COMMAND=$1

if [ "${COMMAND}" = "restore" ]
then
    if ! [ -d ${LOCAL_DIR_RESTORE} ]
    then
	${MKDIR} -p ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
    else
	${RM} -f -r ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
	${MKDIR} -p ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
    fi

    export PASSPHRASE=${DUPLICITY_GPG_PASSPHRASE}
    ${DUPLICITY} ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
    restore_gnupanel_postgres ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
    restore_gnupanel_mysql ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
    restore_gnupanel_files ${LOCAL_DIR_RESTORE} 1>>${LOG_FILE} 2>>${ERR_FILE}
    gnupanel_perms 1>>${LOG_FILE} 2>>${ERR_FILE}
else
    if ! [ -d ${LOCAL_DIR_BACKUP} ]
    then
	${MKDIR} -p ${LOCAL_DIR_BACKUP} 1>>${LOG_FILE} 2>>${ERR_FILE}
    fi

    FILES_DIR=${LOCAL_DIR_BACKUP}/files

    if ! [ -d ${FILES_DIR} ]
    then
	${MKDIR} -p ${FILES_DIR} 1>>${LOG_FILE} 2>>${ERR_FILE}
    fi

    backup_postgres ${LOCAL_DIR_BACKUP} 1>>${LOG_FILE} 2>>${ERR_FILE}

    backup_mysql ${LOCAL_DIR_BACKUP} 1>>${LOG_FILE} 2>>${ERR_FILE}

    ${RSYNC} -q -a -h -P -R --bwlimit=0 -v --delete-before --force ${DIRS} ${FILES_DIR} 1>>${LOG_FILE} 2>>${ERR_FILE}
    ${ECHO} "${DATE_BACKUP}" > ${LOCAL_DIR_BACKUP}/date-backup.txt 2>>${ERR_FILE}
    ${DPKG} --get-selections > ${LOCAL_DIR_BACKUP}/dpkg-packages.txt 2>>${ERR_FILE}


    DAY_NUM=`${DATE} +%u`

    export PASSPHRASE=${DUPLICITY_GPG_PASSPHRASE}

    if [ "${DAY_NUM}" = "${DAY_FULL}" ]
    then
	${DUPLICITY} full --volsize ${VOLSIZE} --allow-source-mismatch ${LOCAL_DIR_BACKUP} ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel 1>>${LOG_FILE} 2>>${ERR_FILE}
	${DUPLICITY} remove-older-than ${REMOVE_OLDER} --force --volsize ${VOLSIZE} --allow-source-mismatch ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel 1>>${LOG_FILE} 2>>${ERR_FILE}
	${DUPLICITY} cleanup --force --volsize ${VOLSIZE} --allow-source-mismatch ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel 1>>${LOG_FILE} 2>>${ERR_FILE}
    else
	${DUPLICITY} --volsize ${VOLSIZE} --allow-source-mismatch ${LOCAL_DIR_BACKUP} ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel 1>>${LOG_FILE} 2>>${ERR_FILE}
	${DUPLICITY} remove-older-than ${REMOVE_OLDER} --force --volsize ${VOLSIZE} --allow-source-mismatch ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel 1>>${LOG_FILE} 2>>${ERR_FILE}
	${DUPLICITY} cleanup --force --volsize ${VOLSIZE} --allow-source-mismatch ${DUP_SCHEME}://${FTP_USER}:${FTP_PASSWORD}@${FTP_SERVER}/gnupanel 1>>${LOG_FILE} 2>>${ERR_FILE}
    fi

    if [ "${DROP_LOCAL_DIR_BACKUP}" = "yes" ]
    then
	${RM} -r -f ${LOCAL_DIR_BACKUP} 1>>${LOG_FILE} 2>>${ERR_FILE}
    fi
fi



